/*
 * Copyright 2010 Nathanaël Cottin - http://www.insane-network.org
 *
 * This file is part of Insane (Instrumented artificial neural environment).
 *
 * Insane is free software: you can redistribute it and/or modify it under the
 * terms of the Lesser GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or any later version.
 *
 * Insane is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Insane. If not, see <http://www.gnu.org/licenses/>.
 */
package insane.io;

import insane.NetworkLayer;
import insane.NeuralNetwork;
import java.io.IOException;
import java.io.PrintWriter;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;



/**
 * Defines an XML neural network saver
 *
 * @author ncottin
 */
public final class XMLNeuralNetworkSaver extends NeuralNetworkSaver {

    @Override
    public void save(final NeuralNetwork network, final PrintWriter writer) throws IOException {
        Element root = new Element("neuralNetwork");
        encode(network, root);
        writeDocument(root, writer);
    }

    /**
     * Populates an XML element with a neural network
     *
     * @param network The neural network to be represented in XML
     * @param root The root XML elemet to hold the neural network information
     */
    public static void encode(final NeuralNetwork network, Element root) {
        NetworkLayer[] layers = network.getLayers();
        root.setAttribute("inputs", String.valueOf(layers[0].getInputs()));
        Element current;
        Element neuron;
        double[][] weights;
        double[] biases;
        int i;
        for (NetworkLayer layer : layers) {
            current = new Element("layer");
            current.setAttribute("activation", layer.getActivationFunction().getIdentifier());
            current.setAttribute("bias", String.valueOf(layer.getBias()));
            weights = layer.getWeights();
            biases = layer.getBiases();
            for (i=0; i<weights.length; ++i) {
                neuron = new Element("neuron");
                neuron.setAttribute("bias", String.valueOf(biases[i]));
                encodeWeights(weights[i], neuron);
                current.addContent(neuron);
            }

            root.addContent(current);
        }
    }

    private static void encodeWeights(double[] values, Element elt) {
        Element current;
        for (double value : values) {
            current = new Element("weight");
            current.setText(String.valueOf(value));
            elt.addContent(current);
        }
    }

    private static void writeDocument(final Element root, final PrintWriter writer) throws IOException {
        XMLOutputter xop = new XMLOutputter(Format.getPrettyFormat());
        xop.output(new Document(root), writer);
        writer.flush();
    }
}